<?xml version="1.0" encoding="UTF-8"?>

<!--
Copyright 2012 Cellule MathDoc (UMS 5638 CNRS & UJF) http://www.mathdoc.fr/

Author : Nicolas Houillon
Version : 1.1

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:mml='http://www.w3.org/1998/Math/MathML'
                xmlns:xlink="http://www.w3.org/1999/xlink"
                exclude-result-prefixes="mml xlink">
    <xsl:param name="parent" select="''" />
    <xsl:param name="namespace" select="''" />
    
    <xsl:template match="/unknown">
      <result>
        <xsl:apply-templates />
      </result>
    </xsl:template>
    
    <xsl:template match="text()[parent::unknown and position() = 1]">
      <xsl:value-of select="substring(., 2)" />
    </xsl:template>
    
    <xsl:template match="@rend[parent::hi]" />
    <xsl:template match="@overflow[parent::mml:math]" />
    <xsl:template match="error" />
    <xsl:template match="noindent" />

    <xsl:template match="formula">
      <xsl:variable name="formulaElement">
        <xsl:choose>
          <xsl:when test="@type = 'display' and $parent != 'article-title' and $parent != 'trans-title'">
            <xsl:text>disp-formula</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>inline-formula</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:element name="{$formulaElement}" namespace="{$namespace}">
        <xsl:element name="alternatives" namespace="{$namespace}">
          <xsl:apply-templates select="mml:math" mode="mml" />
          <xsl:apply-templates select="texmath" />
        </xsl:element>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="texmath">
      <xsl:element name="tex-math" namespace="{$namespace}">
        <xsl:variable name="tagged">
          <xsl:variable name="ns" select="normalize-space()" />
          <xsl:choose>
            <xsl:when test="starts-with($ns, '$') or starts-with($ns, '\[')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{equation}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{equation*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{align}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{align*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{gather}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{gather*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{flalign}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{flalign*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{multline}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{multline*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{alignat}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{alignat*}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{split}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{eqnarray}')">true</xsl:when>
            <xsl:when test="starts-with($ns, '\begin{eqnarray*}')">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:choose>
          <xsl:when test="$tagged = 'true'" />
          <xsl:when test="../@type = 'inline'">$</xsl:when>
          <xsl:when test="../@type = 'display' and (not(../@textype) or ../@textype = 'display' or ../@textype = 'displaymath')">\[</xsl:when>
          <xsl:otherwise>
            <xsl:text>\begin{</xsl:text>
            <xsl:value-of select="../@textype" />
            <xsl:text>}</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:apply-templates />
        <xsl:choose>
          <xsl:when test="$tagged = 'true'" />
          <xsl:when test="../@type = 'inline'">$</xsl:when>
          <xsl:when test="../@type = 'display' and (not(../@textype) or ../@textype = 'display' or ../@textype = 'displaymath')">\]</xsl:when>
          <xsl:otherwise>
            <xsl:text>\end{</xsl:text>
            <xsl:value-of select="../@textype" />
            <xsl:text>}</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@overflow[parent::mml:math]" mode="mml" />
    <xsl:template match="@mode[parent::mml:math]" mode="mml" />
    
    <xsl:template match="@*" mode="mml">
        <xsl:copy />
    </xsl:template>
    
    <xsl:template match="mml:math" mode="mml">
        <xsl:element name="{local-name()}" namespace="http://www.w3.org/1998/Math/MathML">
            <xsl:attribute name="display">
                <xsl:choose>
                  <xsl:when test="ancestor::formula/@type = 'display' and $parent != 'article-title' and $parent != 'trans-title'">block</xsl:when>
                  <xsl:otherwise>inline</xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
            <xsl:apply-templates select="@*" mode="mml" />
            <xsl:apply-templates mode="mml" />
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="mml:mspace/@width[contains(., '.em') or contains(., '.pt')]" mode="mml">
        <xsl:attribute name="width">
            <xsl:value-of select="concat(substring(., 1, string-length() - 2), '0', substring(., string-length() - 1))" />
        </xsl:attribute>
    </xsl:template>
    
    <xsl:template match="node()" mode="mml">
        <xsl:choose>
            <xsl:when test="self::text()">
                <xsl:copy />
            </xsl:when>
            <xsl:otherwise>
                <xsl:element name="{local-name()}" namespace="http://www.w3.org/1998/Math/MathML">
                    <xsl:apply-templates select="@*" mode="mml" />
                    <xsl:apply-templates mode="mml" />
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="hi">
      <xsl:choose>
        <xsl:when test="@rend = 'bold'">
            <xsl:element name="bold" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'it'">
            <xsl:element name="italic" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'underline'">
            <xsl:element name="underline" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'sansserif'">
            <xsl:element name="sans-serif" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'sup'">
            <xsl:element name="sup" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'sc'">
            <xsl:element name="sc" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'tt'">
            <xsl:element name="monospace" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'overline'">
            <xsl:element name="overline" namespace="{$namespace}">
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'large'">
            <xsl:element name="styled-content" namespace="{$namespace}">
                <xsl:attribute name="style">large</xsl:attribute>
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'small'">
            <xsl:element name="styled-content" namespace="{$namespace}">
                <xsl:attribute name="style">small</xsl:attribute>
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:when test="@rend = 'slanted'">
            <xsl:element name="styled-content" namespace="{$namespace}">
                <xsl:attribute name="style">slanted</xsl:attribute>
                <xsl:apply-templates />
            </xsl:element>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>Unknown hi attribute : <xsl:value-of select="@rend" /> </xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="cit">
      <xsl:element name="xref" namespace="{$namespace}">
          <xsl:attribute name="ref-type">bib-ref</xsl:attribute>
          <xsl:attribute name="rid"><xsl:value-of select="ref/@target" /></xsl:attribute>
          <xsl:value-of select="ref" />
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="mbox">
      <xsl:apply-templates />
    </xsl:template>
    
    <xsl:template match="@label[parent::list and parent::list/following-sibling::*[1][self::item]]" />
    <xsl:template match="@type[parent::list]" />
    <xsl:template match="@order[parent::list]" />
    <xsl:template match="@id[parent::list]" />
    <xsl:template match="@id-text[parent::list]" />
    
    <xsl:template match="list">
      <xsl:element name="list" namespace="{$namespace}">
        <xsl:if test="@type[. != '']">
          <xsl:attribute name="list-type">
            <xsl:value-of select="@type[. != '']" />
          </xsl:attribute>
        </xsl:if>
        <xsl:apply-templates />
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="item[parent::list]">
      <xsl:element name="list-item" namespace="{$namespace}">
        <xsl:if test="@id[. != '']">
          <xsl:attribute name="id">
            <xsl:value-of select="@id[. != '']" />
          </xsl:attribute>
        </xsl:if>
        <xsl:if test="preceding-sibling::*[1][self::label]">
          <xsl:element name="label" namespace="{$namespace}">
            <xsl:apply-templates select="preceding-sibling::*[1]" />
          </xsl:element>
          <xsl:apply-templates />
        </xsl:if>
      </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*|node()">
      <xsl:copy>
        <xsl:apply-templates select="@*"/>
        <xsl:apply-templates/>
      </xsl:copy>
    </xsl:template>
</xsl:stylesheet>